function nba = getNba(formula, env, path_to_ltl2ba) 
%getNba Create NBA (nondet Buchi automaton) from LTL formula
%
% :copyright: 2014 by the California Institute of Technology
% :license: BSD 3-Clause, see LICENSE for details

nbaParse = parseNeverClaim(formula, path_to_ltl2ba);
nba = struct();
nba.init = nbaParse.initial_state_ids;
nba.acc = nbaParse.accepting_state_ids;

L = nbaParse.adj;
n = size(L,1);
A = sparse(n,n);
for i=1:n
    for j=1:n
        if ~isempty(L{i,j})
            A(i,j) = 1;
        else
            A(i,j) = 0;
        end
    end
end

% Label matrix gives a set of labels in DNF, i.e., ...
% (a1 && ... && z1) || (a2 && ... && z2) || ...

%% Create flat mapping from AP/notAP to polytopes
ap2poly = struct();
ap2poly.safe = env.work.safe;
ap2poly.('not_safe') = env.work.unsafe;

fields = fieldnames(env.AP);
for i = 1:length(fields)
    ap2poly.(fields{i}) = env.AP.(fields{i});
end

fields = fieldnames(env.notAP);
for i = 1:length(fields)
    ap2poly.(strcat('not_',fields{i})) = env.notAP.(fields{i});
end


%% Convert string representation into placeholders for polytope objects 
L_env = L; 
AP = fieldnames(ap2poly);

[I,J] = find(A);
for k = 1:length(I)
    i = I(k);
    j = J(k);
    label = L(i,j);
    label = label{1};    % (conj1) OR (conj2) OR (conj3) OR ...

    if ~isempty(label)

        % Compute polytope for each disjunction (unions)
        disjPolys = {};
        for indDisj = 1:length(label)
            conjList = label{indDisj};

            % Compute polytope for each conjunction (intersections)
            conjPolys = {};
            for indConj = 1:length(conjList)
                prop = conjList{indConj};
                if strcmp('!', prop(1))
                    prop = strcat('not_', prop(2:end));
                end

                for ind = 1:length(AP)
                    if strcmpi(AP{ind}, prop)
                        conjPolys{indConj} = ap2poly.(AP{ind});
                        break
                    end
                end
            end

            % Intersect all polytopes involed in this conjunction
            conjPoly = conjPolys{1};
            if length(conjPolys) > 1
                for p = 2:length(conjPolys)
                    if dimension(conjPoly) < env.util.dim
                        conjPoly = [];
                        break
                    end
                    conjPoly = intersect(conjPolys{p}, conjPoly);
                end
            end
            disjPolys{indDisj} = conjPoly;
        end

        % Union all polytopes involed in all disjunctions
        disjPoly = disjPolys{1};
        if length(disjPolys) > 1
            for p = 2:length(conjPolys)
                disjPoly = union([disjPolys{p}, disjPoly]);
            end
        end
        L_env{i,j} = disjPoly;

    else
        L_env{i,j} = [];
    end
end

%% Remove NBA edges that cannot be realized simply by checking set intersections
for k = 1:length(I)
    i = I(k);
    j = J(k);

    if length(L_env{i,j}) == 0      % isempty does not work for Polytope
        A(i,j) = 0;
        L_env{i,j} = [];
    end
end

nba.A = A;
nba.L = L_env;

end
