function [prefix,suffix,cost,sol] = getTrajSim(X, U, J, ltl, ...
                                               fSys, fInit, model, sys, options, ...
                                               N, Npre, Nsuf, bigM, auto, blkVar)
% getTrajSim Generate trajectory over prefix and suffix simultaneously
%
% :copyright: 2014 by the California Institute of Technology
% :license: BSD 3-Clause, see LICENSE for details
                                           
C = sys.C;
nx = sys.nx;
nu = sys.nu;

% Compute the prefix and suffix simultaneously
if auto

    if strcmp(model(1:3), 'car')
        error('WARNING! The loop constraint for Dubins car is not yet implemented.');
    end

    % Automatically choose where the trajectory splits
    % between the prefix and suffix.
    tol = 1e-4;

    s = binvar(N+1, 1);
%         F_s = [s(15) == 0, s(16) == 1];
    fS = [s(1) == 0];
    for t=1:N
        fS = [fS, s(t+1) >= s(t)];
    end

    % Enforce the loop closure  (TODO: Improve by removing tolerances)
    for t=2:N+1
        assert(length(X((t-1)*nx+1:t*nx)) == nx);
        fS = [fS, X((t-1)*nx+1:t*nx) <= X(N*nx+1:end) + bigM*ones(nx,1)*(1+s(t-1)-s(t)) + tol, ...
                  X((t-1)*nx+1:t*nx) >= X(N*nx+1:end) - bigM*ones(nx,1)*(1+s(t-1)-s(t)) - tol];
    end

    % Create the appropriate LTL constraints on the prefix and suffix
    fG = safe(ltl.G, C, nx, X, N, bigM, blkVar);
    fF = goal(ltl.F, C, nx, X, N, bigM, blkVar);
    fFG = safeAuto(ltl.FG, C, X, N, bigM, s);
    fGF = goalAuto(ltl.GF, C, X, N, bigM, s);

    % Only enforce the cost function on the infinite suffix
    Us = sdpvar(N*nu, 1);
    for t=1:N
        fS = [fS, Us((t-1)*nu+1:t*nu) <=  bigM*ones(nu,1)*s(t), ...
                  Us((t-1)*nu+1:t*nu) >= -bigM*ones(nu,1)*s(t), ...
                  Us((t-1)*nu+1:t*nu) <= U((t-1)*nu+1:t*nu) + bigM*ones(nu,1)*(1-s(t)) + tol, ...
                  Us((t-1)*nu+1:t*nu) >= U((t-1)*nu+1:t*nu) - bigM*ones(nu,1)*(1-s(t)) - tol];
    end

    F = [fG, fF, fGF, fFG, fSys, fInit, fS];

    sol = solvesdp(F, J, options);
    ctrl = double(U);
    traj = double(X);
    cost = double(J);
    s = double(s)'

    % Find location of split on suffix and prefix.
    Npre = 1;
    for i = 1:length(s)-1
        if s(i) ~= s(i+1)
            Npre = i;
            break
        end
    end
    prefix = traj(1:Npre*nx);
    suffix = traj(Npre*nx+1:end);

else

    % Specify trajectory split between prefix and suffix
    fprintf('Prefix length: %i. Suffix length: %i\n', Npre, Nsuf);
    Xpre = X(1:Npre*nx);
    Xsuf = X(Npre*nx + 1:end);

    fLoop = getLoop(model, Xsuf, nx);

    % Create LTL constraints on the prefix and suffix
    tic
    fG =  safe(ltl.G,  C, nx, X,    N,    bigM, blkVar);
    fFG = safe(ltl.FG, C, nx, Xsuf, Nsuf, bigM, blkVar);

    fF  = goal(ltl.F, C, nx,  Xpre, Npre - 1, bigM, blkVar);
    fGF = goal(ltl.GF, C, nx, Xsuf, Nsuf,     bigM, blkVar);
    toc

    F = [fG, fF, fFG, fGF, fSys, fInit, fLoop];

    sol = solvesdp(F, J, options);
    ctrl = double(U);
    prefix = double(Xpre);
    suffix = double(Xsuf);
    cost = double(J);
end

end
