function [X, U, F, sys] = car3z(dt, N, dim, linear)
% car3z Creates sdpvars and constraints for Dubins car airplane model
%
% Dubins car hybrid model
% x = (x,y,th,z,zd),  u = (uk,uz) curvature (m^-1) and accel (m/s^2)
% x = position (m),   y = position (m), th = orientation (rad), 
% z = height (m),  zd = height velocity (m/s)
% Dynamics: xd= v0*cos(th), yd= v0*sin(th), thd= u_k, zd= zd, zdd= uz
%
% Input: 'dt' timestep (sec)
%        'N' length of traj
%        'dim' workspace dimension (2D or 3D)
%        'linear' parameters for linearization of nonlinear model
% Output: 'X' sdpvar for state
%         'U' sdpvar for control
%         'F' dynamic constraints
%         'sys' system info
%
% :copyright: 2014 by the California Institute of Technology
% :license: BSD 3-Clause, see LICENSE for details


% Parameters
nx = 5; % number of states
nu = 2; % number of inputs

v0 = 0.5; % (m/s)

%% Common constraints
if dim == 3
    sys.C = [1 0 0 0 0;     % x
             0 1 0 0 0;     % y
             0 0 0 1 0];    % z 
elseif dim == 2
    sys.C = [1 0 0 0 0;     % x
             0 1 0 0 0];    % y
else
    error('Invalid dimension.');
end

uk = 1.75*[-1, 1];     %m^-1
uz = 0.25*[-1, 1];  %m/s^2
Hu = [eye(nu); -eye(nu)];
Ku = [uk(2);  uz(2);
     -uk(1); -uz(1)];
sys.Uset = polytope(Hu, Ku);

x = [0, 10];    % m
y = [0, 10];    % m
th = pi*[-1, 1];  % rad
z = [0, 10];     % m
zd = [-1, 1];    % m/s
Hx = [eye(nx); -eye(nx)];
Kx = [ x(2);  y(2);  th(2);  z(2);  zd(2);
      -x(1); -y(1); -th(1); -z(1); -zd(1)];
sys.Xset = polytope(Hx, Kx);


%% Create system model
u = sdpvar(repmat(nu,1, N), ones(1, N));
x = sdpvar(repmat(nx, 1, N+1), ones(1, N+1));

F = [];
if isempty(linear)   % Nonlinear model
    for t = 1:N
        F = [F, x{t+1} == x{t} + dt*[v0*cos(x{t}(3)); 
                                     v0*sin(x{t}(3));
                                     u{t}(1);
                                     x{t}(5);
                                     u{t}(2)], ...
                Hx*x{t} <= Kx, Hu*u{t} <= Ku];
    end
else
    % Linear hybrid automaton model
    blk_mode = linear(1);
    numTH = linear(2);

    th0_vec = linspace(-pi, pi, numTH + 1) + pi/numTH;
    th0_vec = th0_vec(1:end-1)
    th_guard = linspace(-pi, pi, numTH + 1);
    A = {};
    B = {};
    K = {};
    for i = 1:numTH
        th0 = th0_vec(i);

        % Linearized model about th0
        Ac = [0, 0, -v0*sin(th0), 0, 0; 
              0, 0,  v0*cos(th0), 0, 0;
              0, 0, 0, 0, 0;
              0, 0, 0, 0, 1;
              0, 0, 0, 0, 0];

        Bc = [0 0;
              0 0;
              1 0;
              0 0;
              0 1];

        A{i} = eye(nx) + dt*Ac;
        B{i} = dt*Bc;
        K{i} = dt*[ v0*cos(th0) + v0*sin(th0)*th0;
                    v0*sin(th0) - v0*cos(th0)*th0;
                    0;
                    0;
                    0];
    end 

    M = 10*ones(nx, 1);
    assert( blk_mode >= 1 )   
    if blk_mode >=2
        fprintf('\nBlocking modes by %i.\n',blk_mode);
    end

    for t = 1:N
        if mod(t+blk_mode-1,blk_mode) == 0
            z = binvar(numTH,1);
            F = [F, sum(z) == 1];
        end

        for i=1:numTH
            % Constrain modes to certain theta values, i.e.,
            % x{k+1} == A{i}*x{k} + B{i}*u{k} + K{i} iff theta \in [a,b]
            F = [F, x{t+1} <= A{i}*x{t} + B{i}*u{t} + K{i} + M*(1-z(i)), ...
                    x{t+1} >= A{i}*x{t} + B{i}*u{t} + K{i} - M*(1-z(i)), ...
                    x{t}(3) <= th_guard(i+1) + M*(1-z(i)), ...
                    x{t}(3) >= th_guard(i)   - M*(1-z(i))];
        end
        F = [F, Hx*x{t}<=Kx, Hu*u{t}<= Ku];
    end 
end
F = [F, Hx*x{t+1} <= Kx];

X = reshape([x{:}], (N+1)*nx,1);
U = reshape([u{:}], N*nu,1);

sys.dim = dim;
sys.nx = nx; % Number of states
sys.nu = nu; % Number of inputs

end
